@@ -17,7 +17,7 @@ import datetime
 import os
 import sys
 from fabric.api import (hide, local, env, execute, abort, lcd, sudo, run, cd, puts, settings as fab_settings,
-                        quiet, prompt, get)
+                        quiet, prompt, get, prefix)
 from fabric.colors import red, green, yellow, cyan
 from fabric.contrib import django as fab_django
 from fabric.contrib.console import confirm
@@ -40,7 +40,7 @@ env.roledefs.update({
 # Private Functions
 
 
-def deploy_setting(project_name=None, deploy_settings_path=None, **setting_patches):
+def load_deploy_setting(project_name=None, deploy_settings_path=None, **setting_patches):
     """Get deploy_settings.
 
     This method will be called by deploy task.
@@ -82,11 +82,12 @@ def deploy_setting(project_name=None, deploy_settings_path=None, **setting_patch
     patches = {}
     for key, value in setting_patches.items():
         patches[key] = fab_argument(value)
-    env.deploy_settings = SettingsDict(default_deploy_settings_path, deploy_settings_path, **patches)
+    env.deploy_settings_dict = deploy_settings_dict = \
+        SettingsDict(default_deploy_settings_path, deploy_settings_path, **patches)
 
     # Find project name - 2
     if project_name is None:
-        project_name = getattr(env.deploy_settings, 'PROJECT_NAME', None)
+        project_name = getattr(deploy_settings_dict, 'PROJECT_NAME', None)
     if project_name is None:
         error(red('What is your project name?', bold=True))
 
@@ -97,13 +98,14 @@ def deploy_setting(project_name=None, deploy_settings_path=None, **setting_patch
 def load_web_servers():
     if len(env.roledefs['web_servers']) == 0:
         with hide('everything'):
-            execute('server.web_server', *env.deploy_settings.get('WEB_SERVERS'))
-            verbose = env.deploy_settings.get('VERBOSE')
+            execute('server.web_server', *env.deploy_settings_dict.get('WEB_SERVERS'))
+            verbose = env.deploy_settings_dict.get('VERBOSE')
             if verbose != 0:
                 puts('')
     if len(env.roledefs['web_servers']) == 0:
         abort('Where do you want to deploy to?')
 
+
 # Tasks
 
 
@@ -117,9 +119,9 @@ def add_user(role='web_servers', project_name=None, deploy_settings_path=None, *
     if not os.path.exists(ssh_key_path):
         abort('No file found at %s' % ssh_key_path)
 
-    execute(
-        deploy_setting, project_name=project_name, deploy_settings_path=deploy_settings_path, **additional_settings
-    )
+    load_deploy_setting(project_name=project_name, deploy_settings_path=deploy_settings_path, **additional_settings)
+    project_name = env.project_name
+
     execute(load_web_servers)
 
     ssh_home_path = '~/.ssh'
@@ -158,7 +160,7 @@ def add_user(role='web_servers', project_name=None, deploy_settings_path=None, *
 
 
 @task
-def setup_server():
+def setup_server(project_name=None, deploy_settings_path=None, **additional_settings):
     if env.host_string is None:
         abort('Assign a target server you can reach')
 
@@ -170,9 +172,23 @@ def setup_server():
         sudo('useradd eggplant')
         sudo('passwd eggplant')
         sudo('usermod -a -G wheel eggplant')
+        sudo('su -c "git config --global user.name \'eggplant\'" - eggplant')
+        sudo('su -c "git config --global user.email eggplant@wantoto.com" - eggplant')
 
     execute(add_user)
 
+    load_deploy_setting(project_name=project_name, deploy_settings_path=deploy_settings_path, **additional_settings)
+    deploy_settings = env.deploy_settings_dict
+    remote_base_path = deploy_settings.get('REMOTE_BASE')
+
+    # Make room for deploy target
+    puts('Deploy target is: %s' % remote_base_path)
+    if not remote_exists(remote_base_path):
+        print(yellow('Deploy target doesn\'t exist. Create one now.'))
+        sudo('mkdir -p %s' % remote_base_path)
+        sudo('chmod o+rx %s' % remote_base_path)
+        sudo('chown -R eggplant:eggplant %s' % remote_base_path)
+
 
 @task(default=True)
 def deploy(project_name=None, deploy_settings_path=None, commit_msg='deploy', **additional_settings):
@@ -191,7 +207,7 @@ def deploy(project_name=None, deploy_settings_path=None, commit_msg='deploy', **
         The location of "deploy_settings.py". If you don't specify this,
         eggplant will find in following ways until found:
 
-        1. 'env' of current fabric context (env.deploy_settings = <DEPLOY_SETTING_MODULE>)
+        1. 'env' of current fabric context (env.deploy_settings_dict = <DEPLOY_SETTING_MODULE>)
         2. <CURRENT_WORKING_DIRECTORY>/deploy_settings.py
         3. <CURRENT_WORKING_DIRECTORY>/<PROJECT_NAME>/deploy_settings.py
             (If 'project_name' is specified or can be found without deploy_settings)
@@ -206,11 +222,9 @@ def deploy(project_name=None, deploy_settings_path=None, commit_msg='deploy', **
 
     # Get deploy settings and project name
     global gitignores, pyc_pattern, raw_gitignores, project_src_root_gitignore
-    execute(
-        deploy_setting, project_name=project_name, deploy_settings_path=deploy_settings_path, **additional_settings
-    )
+    load_deploy_setting(project_name=project_name, deploy_settings_path=deploy_settings_path, **additional_settings)
     project_name = env.project_name
-    deploy_settings = env.deploy_settings
+    deploy_settings = env.deploy_settings_dict
     env.update_apache_conf = deploy_settings.get('UPDATE_APACHE_CONF')
 
     # Check output level
@@ -427,7 +441,7 @@ def deploy_to_remote():
 
     # Get variables
     remote_base_path = env.remote_base_path
-    deploy_settings = env.deploy_settings
+    deploy_settings = env.deploy_settings_dict
     project_name = env.project_name
     source_root_name = 'source'
     venv_name = 'venv'
@@ -494,21 +508,13 @@ def deploy_to_remote():
             print(yellow('Updating yum packages'))
             target_sudo('yum install -y %s' % yum_package_string)
 
-    # Make room for deploy target
-    puts('Deploy target is: %s' % remote_base_path)
-    if not target_file_existence(remote_base_path):
-        print(yellow('Deploy target doesn\'t exist. Create one now.'))
-        target_sudo('mkdir -p %s' % remote_base_path)
-        target_sudo('chmod o+rx %s' % remote_base_path)
-
     # Init project
     if not target_file_existence(remote_project_path):
         print(yellow('First time deploy, create git repository:'))
         update_apache_conf = True
         # Make room
-        target_sudo('mkdir -p %s' % remote_project_path)
-        target_sudo('mkdir -p %s' % remote_project_log_root)
-        target_sudo('chown -R eggplant:eggplant %s' % remote_project_path)
+        target_run('mkdir -p %s' % remote_project_path)
+        target_run('mkdir -p %s' % remote_project_log_root)
         # Source folder with git
         target_run('mkdir -p %s' % remote_project_source_root)
         with target_cd(remote_project_path):
