@@ -21,6 +21,8 @@ import urllib
 import urlparse
 from django.core.exceptions import ValidationError
 from django.core.validators import validate_email
+from six import callable
+
 
 def valid_email(email):
     try:
@@ -29,6 +31,7 @@ def valid_email(email):
     except ValidationError:
         return False
 
+
 def append_query_string_to_URL(url, data):
     url_parts = list(urlparse.urlparse(url))
     query = dict(urlparse.parse_qsl(url_parts[4]))
@@ -41,20 +44,24 @@ def append_query_string_to_URL(url, data):
 
     return urlparse.urlunparse(url_parts)
 
+
 def sql_string_from_query_set(qs, *args):
     sql = str(qs.query)
     for target, newString in args:
         sql = sql.replace(target, newString)
     return sql
 
+
 def import_from_string(path):
     path_components = path.split('.')
     class_name = path_components[-1]
     package = '.'.join(path_components[:-1])
     return getattr(__import__(package, fromlist=[class_name]), class_name)
 
+
 def dict_from_module(module_object, key_filter=None):
-    if key_filter is None: key_filter = lambda x: True
+    if key_filter is None:
+        key_filter = lambda x: True
 
     result = {}
     for attribute in dir(module_object):
@@ -84,21 +91,29 @@ class SettingsDict(object):
         self._settings = settings
 
     def get(self, *args):
-        if len(args) == 1: return self._get(args[0])
-        elif len(args) == 2: return self._get_with_default(args[0], args[1])
+        if len(args) == 1:
+            return self._get(args[0])
+        elif len(args) == 2:
+            return self._get_with_default(args[0], args[1])
         else:
-            raise TypeError, 'get() takes exactly 1 or 2 arguments (%s given)' % len(args)
+            raise TypeError('get() takes exactly 1 or 2 arguments (%s given)' % len(args))
 
     def _get(self, key):
-        if self._settings.has_key(key):
-            return self._settings[key]
+        if key in self._settings:
+            target = self._settings[key]
+            if callable(target):
+                return target()
+            else:
+                return target
+
         else:
-            raise KeyError, key
+            raise KeyError(key)
 
     def _get_with_default(self, key, default):
-        if self._settings.has_key(key):
-            return self._settings[key]
-        return default
+        try:
+            return self._get(key)
+        except KeyError:
+            return default
 
     def __getattr__(self, item):
-        return self.get(item)
\ No newline at end of file
+        return self.get(item)
