@@ -30,7 +30,7 @@ def temporary_directory(identifier='com.wantoto.eggplant', prefix=None, suffix=N
         os.system('rm -rf %s' % path)
 
 
-def rsync(source, destination, rsync_schema=None, rsync_argument='-rlptzD'):
+def rsync(source, destination, rsync_schema=None, rsync_argument='-rlpctzD'):
     """Using rsync as cp or scp. Note both source and destination do not support ":" (colon)
     :param source:
         path of source. (local path or remote path like scp/rsync)
