@@ -6,7 +6,7 @@ from six import PY3
 
 CURRENT_TZ = timezone(settings.TIME_ZONE)
 
-def utc_timestamp(raw_datetime, use_float_seconds=False):
+def timestamp(raw_datetime, use_float_seconds=False):
     timestamp_format = '%s.%f' if use_float_seconds else '%s'
     if use_float_seconds:
         result_type = float
@@ -14,12 +14,13 @@ def utc_timestamp(raw_datetime, use_float_seconds=False):
         result_type = long if not PY3 else int
 
     if is_naive(raw_datetime):
-        utc_diff_seconds = (datetime.datetime.now() - datetime.datetime.utcnow()).total_seconds()
-        utc_datetime = raw_datetime - datetime.timedelta(seconds=utc_diff_seconds)
+        target_datetime = raw_datetime
     else:
-        utc_datetime = raw_datetime.astimezone(UTC)
+        # Since python's strftime('%s') doesn't respect tzinfo,
+        # we convert it back to our current timezone first.
+        target_datetime = raw_datetime.astimezone(CURRENT_TZ)
 
-    return result_type(utc_datetime.strftime(timestamp_format))
+    return result_type(target_datetime.strftime(timestamp_format))
 
 def time_to_seconds(time_object, use_float_seconds=False):
     seconds = time_object.hour*3600 + time_object.minute*60 + time_object.second
