@@ -14,9 +14,12 @@ See the License for the specific language governing permissions and
 limitations under the License.
 """
 
+import re
 from django.conf import settings
-from django.core.exceptions import ImproperlyConfigured
+from django.core.cache import cache
+from django.core.exceptions import ImproperlyConfigured, FieldError
 from django.core.paginator import Paginator, EmptyPage
+from django.db.models import Q
 import yajl as json
 from eggplant.utils.url import replace_querystring
 from eggplant.validators.number import range_validator
@@ -28,6 +31,26 @@ from eggplant.views.decorators import validate_keys
 from eggplant.constants.settings import MAX_PAGINATION_SIZE, DEFAULT_PAGINATION_SIZE
 from eggplant.constants import defaults as eggplant_defaults
 
+FILTER_CACHE_KEY_PREFIX = 'com.wantoto.eggplant.views.api.model.filter'
+ORDER_CACHE_KEY_PREFIX  = 'com.wantoto.eggplant.views.api.model.order'
+
+SUPPORTED_FILTER_TYPES = [
+    ('=',  '__exact'),
+    ('i=', '__iexact'),
+    ('>',  '__gt'),
+    ('<',  '__lt'),
+    ('>=', '__gte'),
+    ('<=', '__lte'),
+    ('%',  '__contains'),
+    ('i%', '__icontains'),
+    ('^',  '__startswith'),
+    ('i^', '__istartswith'),
+    ('$',  '__endswith'),
+    ('i$', '__iendswith'),
+    ('@', '__in'),
+    ]
+SUPPORTED_FILTER_TYPES_DICT = dict(SUPPORTED_FILTER_TYPES)
+
 #-----------------------------------------------------------------------------------------------------------------------
 
 class ModelViewBase(APIView):
@@ -57,6 +80,10 @@ class ModelViewBase(APIView):
 class ModelCollectionView(ModelViewBase):
     MODEL_COLLECTION_CLASS = None
     ORDERABLE_FIELDS = []
+
+    SKIP_DELIMITER_IN_FILTERS = False
+    VALID_FILTERS_FOR_FIELDS = {}
+
     DEFAULT_PAGE_SIZE = getattr(settings, DEFAULT_PAGINATION_SIZE, eggplant_defaults.EGGPLANT_DEFAULT_PAGINATION_SIZE)
 
     #-------------------------------------------------------------------------------------------------------------------
@@ -87,10 +114,97 @@ class ModelCollectionView(ModelViewBase):
         # Get query set
         query_set = self.queryset(*args, **kwargs)
 
+        # Filter
+        # "field:operation:value" with ;&| as connector
+        # For example:
+        #    name:=:broadcast;owner:=:sodas ==> name == broadcast AND owner == sodas
+        #    name:=:broadcast&owner:=:sodas ==> name == broadcast AND owner == sodas
+        #    name:=:broadcast|owner:=:sodas ==> name == broadcast OR owner == sodas
+        #    name:=:broadcast;owner:=:sodas&year:=:2012 ==> name == broadcast AND (owner == sodas AND year == 2012)
+        #    name:=:broadcast;owner:=:sodas|year:=:2012 ==> name == broadcast AND (owner == sodas OR year == 2012)
+        #    name:=:broadcast;owner:=:sodas;year:=:2012 ==> name == broadcast AND owner == sodas AND year == 2012
+        raw_filter = self.request.GET.get('filter', None)
+        filters = None
+        filter_cache_key = None
+        if raw_filter is not None:
+            # Get from cache
+            filter_cache_key = '%s.%s'%(FILTER_CACHE_KEY_PREFIX, raw_filter)
+            filters = cache.get(filter_cache_key)
+        # Make new if cache is empty
+        if raw_filter is not None and filters is None:
+            filters = []
+            # RE: LookAhead and LookBehind (?=...) (?!...), (?<=...) (?<!...)
+            # Break ";" first
+            and_group = re.split(r'(?<!\\);', raw_filter) if self.SKIP_DELIMITER_IN_FILTERS else raw_filter.split(';')
+            for and_pair in and_group:
+                # Get components separated by '&' or '|'
+                # and_pair ==> 'field:op:value&field:op:value'
+                # query_components ==> ['field:op:value', '&', 'field:op:value']
+                # lookups ==> ['field:op:value', 'field:op:value']
+                # connectors ==> ['&']
+                if self.SKIP_DELIMITER_IN_FILTERS:
+                    query_components = [item.replace('\\', '') for item in re.split(r'((?<!\\)&|(?<!\\)\|)', and_pair)]
+                else:
+                    query_components = re.split(r'(&|\|)', and_pair)
+                lookups = query_components[::2] # a,b,c ==> a,c
+                connectors = query_components[1::2] # a,b,c ==> b
+
+                # Create each Q objects separated by '&' or '|'
+                q_objects = []
+                for lookup in lookups:
+                    # Component
+                    try:
+                        field, filter_type, value = \
+                            re.split(r'(?<!\\):', lookup) if self.SKIP_DELIMITER_IN_FILTERS else lookup.split(':')
+                    except ValueError:
+                        raise exceptions.BadRequestError('Error filter format: %s'%lookup)
+
+                    # Field name and negate
+                    negate = field.startswith('~')
+                    if negate: field = field[1:]
+
+                    # Check field name and filter type
+                    if not self.VALID_FILTERS_FOR_FIELDS.has_key(field):
+                        continue # Not a valid field
+                    valid_filter_types = self.VALID_FILTERS_FOR_FIELDS[field]
+                    if filter_type not in valid_filter_types:
+                        continue # not a valid filter type
+
+                    # Is value a list?
+                    if filter_type=='@':
+                        value = re.split(r'(?<!\\),', value) if self.SKIP_DELIMITER_IN_FILTERS else value.split(',')
+
+                    # Create object
+                    q_object = self.q_object_from_filter(field, filter_type, value, negate)
+                    if q_object is not None: q_objects.append(q_object)
+
+                # Connect each Q objects in a ',' via & or |
+                final_q_object = None
+                for index, q in enumerate(q_objects):
+                    if index==0: final_q_object = q
+                    else:
+                        if connectors[index-1]=='&': final_q_object &= q
+                        elif connectors[index-1]=='|': final_q_object |= q
+                if final_q_object is not None: filters.append(final_q_object)
+            # Write back to cache
+            cache.set(filter_cache_key, filters)
+        # Apply filter to query set
+        try:
+            if filters is not None and len(filters)!=0: query_set = query_set.filter(*filters)
+        except FieldError, e:
+            raise ImproperlyConfigured('Field is not valid: %s'%e.message)
+        except ValueError:
+            raise exceptions.BadRequestError('Error filter value')
+
         # Order
         raw_order = self.request.GET.get('order', None)
-        if raw_order:
-            orders = []
+        order_cache_key = None
+        orders = None
+        if raw_order is not None:
+            order_cache_key = '%s.%s'%(ORDER_CACHE_KEY_PREFIX, raw_order)
+            orders = cache.get(order_cache_key)
+        if raw_order is not None and orders is None:
+            order_pairs = []
             raw_orders = raw_order.split(',')
             for raw_order in raw_orders:
                 if raw_order.startswith('-'):
@@ -100,24 +214,13 @@ class ModelCollectionView(ModelViewBase):
                     order = raw_order
                     direction = 1
                 if order in self.ORDERABLE_FIELDS:
-                    orders += [(direction, order)]
-            if len(orders):
-                query_set = self.order(query_set, orders, *args, **kwargs)
-
-        # Filter
-#        raw_filter = self.request.GET.get('filter', None)
-#        if raw_filter:
-#            try: filters = json.loads(raw_filter)
-#            except ValueError: filters = []
-#        else: filters = []
-#        for filter in filters:
-#            try:
-#                key, operator, value = filter
-#                # TODO: do filter
-#            except ValueError:
-#                e = exceptions.BadRequestError('Invalid filter format')
-#                e.user_info = {'raw_filter': filter}
-#                raise e
+                    order_pairs.append((direction, order))
+            orders = self.order_list(query_set, order_pairs, *args, **kwargs)
+            cache.set(order_cache_key, orders)
+        try:
+            if orders is not None and len(orders)!=0: query_set = query_set.order_by(*orders)
+        except FieldError, e:
+            raise ImproperlyConfigured('Field is not valid: %s'%e.message)
 
         meta_info = {}
         # Pagination
@@ -239,15 +342,21 @@ class ModelCollectionView(ModelViewBase):
             raise ImproperlyConfigured, 'Forget to set MODEL_CLASS for this ModelCollectionView'
         return ModelClass.objects.all()
 
-    def order(self, query_set, orders, *args, **kwargs):
+    def order_list(self, query_set, orders, *args, **kwargs):
         result_order_list = []
         for direction, raw_field in orders:
             order = self.field_for_order(raw_field, direction)
+            if order is None: continue
             if isinstance(order, (list, tuple)):
                 result_order_list += order
             else:
-                result_order_list += [order]
-        return query_set.order_by(*result_order_list)
+                result_order_list.append(order)
+        return  result_order_list
+
+    def q_object_from_filter(self, field, filter_type, value, negate):
+        q = Q(**{field+SUPPORTED_FILTER_TYPES_DICT[filter_type]: value})
+        if negate: return ~q
+        else: return q
 
     def field_for_order(self, order, direction, *args, **kwargs):
         return order if direction==1 else '-'+order
