@@ -21,30 +21,31 @@ else:
     import httplib
 
 class ViewError(Exception):
-    def __init__(self, status_code, message=None):
+    def __init__(self, status_code, message=None, **kwargs):
         self.status_code = status_code
         self.reason = httplib.responses[status_code]
         self.message = message or 'Something goes wrong ...'
         self.error_handler = None
         self.user_info = None
+        self.init_kwargs = kwargs
 
 #-----------------------------------------------------------------------------------------------------------------------
 # 400 Bad Request
 # Request syntax error (Lack of required keys, invalid value of arguments)
 
 class BadRequestError(ViewError):
-    def __init__(self, message=None):
-        super(BadRequestError, self).__init__(httplib.BAD_REQUEST, message or 'Bad Request')
+    def __init__(self, message=None, **kwargs):
+        super(BadRequestError, self).__init__(httplib.BAD_REQUEST, message or 'Bad Request', **kwargs)
 
 class LackOfRequiredKeyError(BadRequestError):
-    def __init__(self, key):
-        super(LackOfRequiredKeyError, self).__init__('Key "%s" is required.'%key)
+    def __init__(self, key, **kwargs):
+        super(LackOfRequiredKeyError, self).__init__('Key "%s" is required.'%key, **kwargs)
         self.user_info = {'key': key}
 
 class KeyValidationError(BadRequestError):
-    def __init__(self, key, messages):
+    def __init__(self, key, messages, **kwargs):
         message = ('Key "%s" is invalid.' % key + ' ' + ' '.join(messages)).strip()
-        super(KeyValidationError, self).__init__(message)
+        super(KeyValidationError, self).__init__(message, **kwargs)
         self.user_info = {
             'key': key,
             'raw_messages': messages
@@ -55,66 +56,67 @@ class KeyValidationError(BadRequestError):
 # Failed to authenticate user. Credential error or lack of required credential
 
 class UnauthorizedError(ViewError):
-    def __init__(self, message=None):
+    def __init__(self, message=None, **kwargs):
         super(UnauthorizedError, self).__init__(httplib.UNAUTHORIZED,
-            message or 'You have to login before access this resources.')
+            message or 'You have to login before access this resources.', **kwargs)
 
 class SignatureKeyInvalidError(UnauthorizedError):
-    def __init__(self):
-        super(SignatureKeyInvalidError, self).__init__('The signature key is invalid.')
+    def __init__(self, **kwargs):
+        super(SignatureKeyInvalidError, self).__init__('The signature key is invalid.', **kwargs)
 
 class UserIdentityUnknownError(UnauthorizedError):
-    def __init__(self):
-        super(UserIdentityUnknownError, self).__init__('Your identity is unknown.')
+    def __init__(self, **kwargs):
+        super(UserIdentityUnknownError, self).__init__('Your identity is unknown.', **kwargs)
 
 #-----------------------------------------------------------------------------------------------------------------------
 # 403 Forbidden
 # Authenticated but don't have enough permission
 
 class ForbiddenError(ViewError):
-    def __init__(self, message=None):
+    def __init__(self, message=None, **kwargs):
         super(ForbiddenError, self).__init__(httplib.FORBIDDEN,
-            message or 'You have no permission to access this resources.')
+            message or 'You have no permission to access this resources.', **kwargs)
 
 class InvalidHttpRefererError(ForbiddenError):
-    def __init__(self):
-        super(InvalidHttpRefererError, self).__init__('This http refer of this request is not accepted.')
+    def __init__(self, **kwargs):
+        super(InvalidHttpRefererError, self).__init__('This http refer of this request is not accepted.', **kwargs)
 
 class AppHaveNoPermissionError(ForbiddenError):
-    def __init__(self):
-        super(AppHaveNoPermissionError, self).__init__('This application doesn\'t have permission to access this.')
+    def __init__(self, **kwargs):
+        super(AppHaveNoPermissionError, self).__init__(
+            'This application doesn\'t have permission to access this.', **kwargs)
 
 class UserHaveNoPermissionError(ForbiddenError):
-    def __init__(self):
-        super(UserHaveNoPermissionError, self).__init__('You have no permission to access this.')
+    def __init__(self, **kwargs):
+        super(UserHaveNoPermissionError, self).__init__('You have no permission to access this.', **kwargs)
 
 #-----------------------------------------------------------------------------------------------------------------------
 # 404 Not Found
 
 class NotFoundError(ViewError):
-    def __init__(self, message=None):
-        super(NotFoundError, self).__init__(httplib.NOT_FOUND, message or 'Item not found')
+    def __init__(self, message=None, **kwargs):
+        super(NotFoundError, self).__init__(httplib.NOT_FOUND, message or 'Item not found', **kwargs)
 
 class ModelDoesNotExistError(NotFoundError):
-    def __init__(self, model_name):
-        super(ModelDoesNotExistError, self).__init__('No such %s item.'%model_name)
+    def __init__(self, model_name, **kwargs):
+        super(ModelDoesNotExistError, self).__init__('No such %s item.'%model_name, **kwargs)
 
 #-----------------------------------------------------------------------------------------------------------------------
 # 405 Method Not Allowed
 
 class MethodNotAllowedError(ViewError):
-    def __init__(self, http_request, message=None):
+    def __init__(self, http_request, message=None, **kwargs):
         super(MethodNotAllowedError, self).__init__(httplib.METHOD_NOT_ALLOWED,
-            message or 'HTTP %s Method is not allowed.'%http_request.method.upper())
+            message or 'HTTP %s Method is not allowed.'%http_request.method.upper(), **kwargs)
 
 #-----------------------------------------------------------------------------------------------------------------------
 # 409 Conflict
 
 class ConflictError(ViewError):
-    def __init__(self, message=None):
-        super(ConflictError, self).__init__(httplib.CONFLICT, message or 'Conflict')
+    def __init__(self, message=None, **kwargs):
+        super(ConflictError, self).__init__(httplib.CONFLICT, message or 'Conflict', **kwargs)
 
 class DuplicatedInsertionError(ConflictError):
-    def __init__(self, message=None):
+    def __init__(self, message=None, **kwargs):
         super(DuplicatedInsertionError, self).__init__(
-            message or 'Requested insertion contains duplicated value for unique keys.')
+            message or 'Requested insertion contains duplicated value for unique keys.', **kwargs)
